"""Top-level package for c*ls: ls wrapper."""

__author__ = """Alastair Irvine"""
__email__ = 'alastair@plug.org.au'
__version__ = '1.2.0'



# *** DEFINITIONS ***
size_col_width = 12
ownership_col_width = 11
